import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JApplet;
import javax.swing.BorderFactory;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
//import java.util.*;
//import java.awt.event.*;

import wj.space.NObject;
import wj.space.NSpace;
import wj.math.shapes.Tetraeder;
import wj.math.shapes.Hexaeder;
import wj.math.shapes.Octaeder;
import wj.math.shapes.Plane;
import wj.math.NVector;

public class Main2 extends JApplet {

	DrawingBoard _drawing;
	DrawControls _controls;

	NObject[] _objects;

	public static void main(String args[])
	{
		JFrame f = new JFrame("universe");
		Main2 mainApplet = new Main2();
		mainApplet.init();
		mainApplet.start();

		f.addWindowListener(
				new WindowAdapter() {
					public void windowClosing(WindowEvent e) {
						System.exit(0);
					}
				}
			);

		f.getContentPane().add(mainApplet);
		f.setSize(new Dimension(640, 480));
		f.show();
	}
	public void init()
	{
		buildFrame();

		_objects = buildObjects();
	}

 
	void buildFrame()
	{
		_drawing = new DrawingBoard(this);
		_controls = new DrawControls(this);

		getContentPane().add(_drawing, BorderLayout.CENTER);
		getContentPane().add(_controls, BorderLayout.SOUTH);
	}

	NObject[] buildObjects()
	{
		NObject[] objects = new NObject[3];

		objects[0] = new NObject(Tetraeder.make(3),new NVector(50,50,50), 50);
		objects[1] = new NObject(Hexaeder.make(3),new NVector(150,150,150), 50);
		objects[2] = new NObject(Octaeder.make(3),new NVector(250,250,250), 50);

		return objects;
	}

	public NSpace getSpace()
	{
		NSpace s = new NSpace(3);

		s.setViewPoint(new NVector(_controls.getx(), _controls.gety(), _controls.getz()), _controls.gets());

		s.setProjectionPlane(
				new Plane(
						new NVector(_controls.getx(),0,0),
						new NVector(0,_controls.gety(),0),
						new NVector(0,0,_controls.getz())));

		s.setSize(_drawing.getw(), _drawing.geth());

		return s;
	}

	public NObject[] getObjects()
	{
		return _objects;
	}
}

class DrawingBoard extends JPanel {
	
	Main2 _main;
	DrawingBoard(Main2 m)
	{
		_main = m;
	}
	public void paintComponent(Graphics g)
	{
		NSpace s = _main.getSpace();
		NObject[] o = _main.getObjects();

		super.paintComponent(g);

		g.setColor(getForeground());
		for (int i=0 ; i<o.length ; i++)
			if (o[i]!=null)
					o[i].drawIn(s, g);
	}
	int getw()
	{
		Insets insets = getInsets();
		return getWidth() - insets.left - insets.right;
	}
	int geth()
	{
		Insets insets = getInsets();
		return getHeight() - insets.top - insets.bottom;
	}
}


class DrawControls extends JPanel implements ChangeListener {
	
	JSlider _xslide;
	JSlider _yslide;
	JSlider _zslide;
	JSlider _sslide;

	Main2 _main;

	DrawControls(Main2 m)
	{
		_main = m;

		buildForm();
	}
	JSlider makeSlider()
	{
		JSlider slide = new JSlider(JSlider.HORIZONTAL,0,500,500);

		slide.addChangeListener(this);
		slide.setMajorTickSpacing(250);
		slide.setMinorTickSpacing(50);
		slide.setPaintTicks(true);
		slide.setPaintLabels(true);
		slide.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));

		return slide;
	}
	public void stateChanged(ChangeEvent e)
	{
		_main.repaint(100);
	}
	void buildForm()
	{
		_sslide = makeSlider();
		_xslide = makeSlider();
		_yslide = makeSlider();
		_zslide = makeSlider();

		GridBagConstraints c = new GridBagConstraints();

		setBorder(BorderFactory.createEmptyBorder(30, 30, 10, 30));
		setLayout(new GridBagLayout());

		c.anchor = GridBagConstraints.NORTHWEST;
		c.gridwidth = 1;
		add(_xslide, c);

		c.anchor = GridBagConstraints.NORTHWEST;
		c.gridwidth = 1;
		add(_yslide, c);

		c.anchor = GridBagConstraints.NORTHWEST;
		c.gridwidth = GridBagConstraints.REMAINDER;
		add(_zslide, c);

	}
	int getx()
	{
		return _xslide.getValue();
	}
	int gety()
	{
		return _yslide.getValue();
	}
	int getz()
	{
		return _zslide.getValue();
	}
	int gets()
	{
		return _sslide.getValue();
	}
}
