package wj.math;

import java.lang.Math;
import wj.math.NVector;

/***********************************************************************
	This represents a matrix with m columns, and n rows
 */
public class Matrix implements Cloneable {
// matrix types
	public static final int NUL = 0;
	public static final int IDENTITY = 1;
// orientation types
	public static final int ROWS = 0;
	public static final int COLUMNS = 1;
/*
	public Object clone()
	{
		try {
			return super.clone();
		} catch(CloneNotSupportedException e) {
			throw new InternalError();
		}
	}
*/
	public Matrix() throws MatrixException
	{
		throw new MatrixException("no default constructor");
	}
	public Matrix(Matrix m)
	{
		a = new double[m.rows()][m.columns()];
		
		for (int row=0 ; row<this.rows() ; row++)
			for (int col=0 ; col<this.columns() ; col++)
				a[row][col]=m.get(row, col);
	}
	public Matrix(int matrixType, int rows, int columns) throws MatrixException
	{
		a = new double[rows][columns];

		if (matrixType==NUL)
		{
			return;
		}
		else if (matrixType==IDENTITY)
		{
			if (rows!=columns)
				throw new MatrixException("matrix is not square!");
			for (int i=0 ; i<rows ; i++)
			{
				a[i][i]=1;
			}
		}
		else
		{
			throw new MatrixException("unknown matrix type");
		}
	}
	public static Matrix newRotation(double phi, int n, int x0, int x1)
	{
		Matrix R = new Matrix(Matrix.IDENTITY,n,n);
		R.set(x0,x0, Math.cos(phi));
		R.set(x0,x1, -Math.sin(phi));
		R.set(x1,x1, Math.cos(phi));
		R.set(x1,x0, Math.sin(phi));
		return R;
	}

/**
	make matrix from array of array of doubles
	m[row][col]
*/
	public Matrix(double[][] m)
	{
		a = new double[m.length][m[0].length];
		for (int row=0 ; row<this.rows() ; row++)
			for (int col=0 ; col<this.columns() ; col++)
				a[row][col]=m[row][col];
	}
/**
	make 1 column matrix from vector
*/
	public Matrix(double[] v)
	{
		a = new double[v.length][1];
		for (int row=0 ; row<this.rows() ; row++)
			a[row][0]=v[row];
	}

/**
	turn array of vectors into matrix
*/
	public Matrix(int orientationType, NVector v[])
	{
		makeMatrix(orientationType, v);
	}
/**
	turn vector into row or column matrix
*/
	public Matrix(int orientationType, NVector v0)
	{
		makeMatrix(orientationType, new NVector[]{v0});
	}
/**
	turn 2 vectors into matrix of 2 colums or 2 rows
*/
	public Matrix(int orientationType, NVector v0, NVector v1)
	{
		makeMatrix(orientationType, new NVector[]{v0, v1});
	}
/**
	turn 3 vectors into matrix of 3 colums or 3 rows
*/
	public Matrix(int orientationType, NVector v0, NVector v1, NVector v2)
	{
		makeMatrix(orientationType, new NVector[]{v0, v1, v2});
	}
/**
	turn 4 vectors into matrix of 4 colums or 4 rows
*/
	public Matrix(int orientationType, NVector v0, NVector v1, NVector v2, NVector v3)
	{
		makeMatrix(orientationType, new NVector[]{v0, v1, v2, v3});
	}

/**
	turn n vectors into matrix of n colums or n rows
*/
	private void makeMatrix(int orientationType, NVector v[]) throws MatrixException
	{
		if (orientationType == ROWS)
		{
			a = new double[v.length][v[0].dimension()];
			for (int row=0 ; row<this.rows() ; row++)
				for (int col=0 ; col<this.columns() ; col++)
				{
					a[row][col] = v[row].get(col);
				}
		}
		else if (orientationType == COLUMNS)
		{
			a = new double[v[0].dimension()][v.length];
			for (int row=0 ; row<this.rows() ; row++)
				for (int col=0 ; col<this.columns() ; col++)
				{
					a[row][col] = v[col].get(row);
				}
		}
		else
		{
			throw new MatrixException("invalid orientation type");
		}
	}
	public int[] dimension()
	{
		return new int[]{this.rows(), this.columns()};
	}
	public int rows()
	{
		return a.length;
	}
	public int columns()
	{
		return a[0].length;
	}

	public void set(int row, int col, double value)
	{
		a[row][col] = value;
	}
	public double get(int row, int col)
	{
		return a[row][col];
	}
	public void setColumn(int col, NVector v) throws MatrixException
	{
		if (v.dimension() != this.rows())
			throw new MatrixException("size mismatch");
		for (int row=0 ; row<this.rows() ; row++)
			this.set(row, col, v.get(row));
	}
	public NVector getColumn(int col)
	{
		NVector result = new NVector(this.rows());

		for (int row=0 ; row<this.rows() ; row++)
			result.set(row, this.get(row, col));

		return result;
	}
	public void setRow(int row, NVector v) throws MatrixException
	{
		if (v.dimension() != this.columns())
			throw new MatrixException("size mismatch");
		for (int col=0 ; col<this.columns() ; col++)
			this.set(row, col, v.get(col));
	}
	public NVector getRow(int row)
	{
		NVector result = new NVector(this.columns());

		for (int col=0 ; col<this.columns() ; col++)
			result.set(col, this.get(row, col));

		return result;
	}

	public void exchangeRows(int r1, int r2)
	{
// NVector tmp=this.getRow(r1)
// this.setRow(r1, this.getRow(r2))
// this.setRow(r2, tmp)

		for (int col=0 ; col<this.columns() ; col++)
		{
			double t=a[r1][col];
			a[r1][col]=a[r2][col];
			a[r2][col]=t;
		}
	}

	public void exchangeColumns(int c1, int c2)
	{
// NVector tmp=this.getColumn(c1)
// this.setColumn(c1, this.getColumn(c2))
// this.setColumn(c2, tmp)
		for (int row=0 ; row<this.rows() ; row++)
		{
			double t=a[row][c1];
			a[row][c1]=a[row][c2];
			a[row][c2]=t;
		}
	}

	public Matrix transpose()
	{
// for (int row=0 ; row<this.rows() ; row++)
//    res.setColumn(row, this.getRow(row))

		Matrix res = new Matrix(Matrix.NUL, columns(), rows());

		for (int row=0 ; row<this.rows() ; row++)
			for (int col=0 ; col<this.columns() ; col++)
				res.set(col, row, a[row][col]);

		return res;
	}

	public void addRowToRow(int r1, int r2, double factor)
	{
		for (int col=0 ; col<this.columns() ; col++)
		{
			a[r1][col] += factor * a[r2][col];
		}
	}

	public void addColumnToColumn(int c1, int c2, double factor)
	{
		for (int row=0 ; row<this.rows() ; row++)
		{
			a[row][c1] += factor * a[row][c2];
		}
	}

/***********************************************************************
	this multiplies this matrix with matrix m

	this:	(trow x tcol)
	m:		(mrow x mcol)
	res:	(rrow x rcol)

	-> mrow should equal tcol
	-> rrow = trow, rcol = mcol
 */
	public Matrix mult(Matrix m) throws MatrixException
	{
		if (m.rows() != this.columns())
		{
			throw new MatrixException("invalid matrix multiplication this=\n" + this.toStr() + "\nm=\n"+m.toStr());
		}

		Matrix res = new Matrix(Matrix.NUL, this.rows(), m.columns());

		for (int row=0 ; row<res.rows() ; row++)
			for (int col=0 ; col<res.columns() ; col++)
			{
// val = this.getRow(row).innerproduct(m.getColum(col))
				double val=0;
				for (int i=0 ; i<this.columns() ; i++)
					val += this.get(row, i) * m.get(i, col);
				res.set(row, col, val);
			}
		return res;
	}
	public NVector mult(NVector v) throws MatrixException
	{
		if (v.dimension() != this.columns())
		{
			throw new MatrixException("size mismatch");
		}

		NVector res = new NVector(this.rows());

		for (int i=0 ; i<res.dimension() ; i++)
		{
			res.set(i, this.getRow(i).innerproduct(v));
		}
		return res;
	}

	public String toStr()
	{
		return "("+rows()+"x"+columns()+")";
	}

	public String toString()
	{
		String res = "[";

		for (int row=0 ; row < rows() ; row++)
		{
			res += "{";
			for (int col=0 ; col < columns() ; col++)
			{
				res += " " + get(row, col);
			}
			res += " }\n";
		}
		res += "]\n";
		return res;
	}

// this algorithm was taken from 
//		'The Art Of Computer Programming' by D.E.Knuth
//		section 4.6.1-exersice 19 - algorithm T
//  C = this,  Y = b
// see also any book on linear algebra
//
//
//		C is a m x n matrix ( m rows, n columns )
//		-> finds U and V (m x m matrices)
//		such that	U * V = I
//					T = V * C = upper triangular
//
// Rn is the space of n-vectors
// C is a matrix with m columns and n rows
//		the rank r of C is the nr of non-zero rows in T
//
// the row space of C: the subspace R(C') of Rn, = { A'*x, x in Rm }
// the column space of C: the subspace R(C) of Rm, = { A*y, y in Rn }
// the null space of C: the subspace N(C) of Rn, = { x, A*x=0 }
// the null space of C': the subspace N(C') of Rm, = { y, A'*y=0 }
// -> C(C) = R(C')
// dim R(C) = r
// dim R(C') = r
// dim N(C) = n-r
// dim N(C') = m-r
//
// N(C) is orthogonal to R(C')
// N(C') is orthogonal to R(C)
//

	private void LU_decomposition()
	{
// important to clone this first, otherwise also V and T will be copied
		T= new Matrix(this);

		Matrix U = new Matrix(Matrix.IDENTITY, this.rows(), this.rows());
		V = new Matrix(Matrix.IDENTITY, this.rows(), this.rows());
		

/* target: T = VC = upper triangular */
		for (int col = 0 ; col<Math.min(T.rows(), T.columns()) ; col++)
		{
			/* zero out column col */
			while (true)
			{
				int k=-1;
				/* find lowest non-zero entry */
				for (int row=T.rows()-1 ; row>=0 ; row--)
					if (!Calculus.isZero(T.get(row, col)))
					{
						k = row;
						break;
					}
				if (k <= col)		// T is upper triangular
					break;

				/* find entry with lowest absolute non-zero value */
				double minVal=Math.abs(T.get(k, col));
				for (int row = k ; row>=0 ; row--)
					if (Calculus.isInOpenInterval( Math.abs(T.get(row, col)), 0, minVal) )
					{
						k = row;
						minVal = Math.abs(T.get(k, col));
					}


				T.exchangeRows(k, col);
				V.exchangeRows(k, col);
				U.exchangeColumns(k, col);
				for (int row = col+1 ; row<T.rows() ; row++)
				{
					double factor = T.get(row, col) / T.get(col, col);

					T.addRowToRow(row, col, -factor);
					V.addRowToRow(row, col, -factor);
					U.addColumnToColumn(col, row, factor);
				}
			}
		}
// now U*T = C, U*V=V*U=I, V*C = T
// to solve C*x=y -> T*x=V*y
	}

/***********************************************************************
	x is een c - vector
	y is een r - vector
	T is een r keer c matrix
	V is een r keer r matrix
	C is een r keer c matrix

	this solves C*X=Y
 */
	public NVector solveAx_b(NVector b)
	{
		if (T==null && V==null) LU_decomposition();

		if (T.rows() < T.columns())
		{
			throw new MatrixException("cannot solve equation for "+T.toStr()+" matrix");
		}
		NVector X = new NVector(T.rows());

		NVector temp = V.mult(b);

		for (int row=T.columns()-1 ; row>=0 ; row--)
		{
			double sum=0;
			for (int col = T.columns()-1 ; col > row ; col--)
				sum += T.get(row, col) * X.get(col);
			X.set(row, (temp.get(row) - sum) / T.get(row, row) );
		}
		return X;
	}

	private Matrix V = null;
	private Matrix T = null;

	private double[][] a;
}
