package wj.math;

import java.lang.Math;

public class NVector implements Cloneable {
/*
	public Object clone()
	{
		try {
			return super.clone();
		} catch(CloneNotSupportedException e) {
			throw new InternalError();
		}
	}
*/
/**
	create nul-vector
*/
	public NVector(int dim)
	{
		_x = new double[dim];
	}
/**
	create unit vector
*/
	public NVector(int dim, int n) throws VectorException
	{
		_x = new double[dim];

		checkIndex(n);

		_x[n]=1;
	}
/**
	create vector from array
*/
	public NVector(double[] v)
	{
		_x=v;
	}

/**
	create 1-vector 
*/
	public NVector(double x0)
	{
		_x = new double[1];
		_x[0] = x0;
	}
/**
	create 2-vector
*/
	public NVector(double x0, double x1)
	{
		_x = new double[2];
		_x[0] = x0;
		_x[1] = x1;
	}
/**
	create 3-vector
*/
	public NVector(double x0, double x1, double x2)
	{
		_x = new double[3];
		_x[0] = x0;
		_x[1] = x1;
		_x[2] = x2;
	}
/**
	create 4-vector
*/
	public NVector(double x0, double x1, double x2, double x3)
	{
		_x = new double[4];
		_x[0] = x0;
		_x[1] = x1;
		_x[2] = x2;
		_x[3] = x3;
	}

	private void checkIndex(int n) throws VectorException
	{
		if (n<0 || n>=this.dimension())
			throw new VectorException("invalid index");
	}
	private void checkDimension(NVector v) throws VectorException
	{
		if (v.dimension() != this.dimension())
			throw new VectorException("vectors do not have the same dimension");
	}

	public int dimension()
	{
		return _x.length;
	}
	public void set(int i, double value)
	{
		checkIndex(i);

		_x[i]=value;
	}
	public double get(int i)
	{
		checkIndex(i);

		return _x[i];
	}
	public double[] coordinates()
	{
		double[] res = new double[this.dimension()];
		for (int i=0 ; i<this.dimension() ; i++)
			res[i] = this.get(i);

		return res;
	}

// ---------------------------------------------------------------------
// methods that return a result NVector, and leave the this NVector unchanged
	public NVector substract(NVector v)
	{
		checkDimension(v);

		NVector result = new NVector(this.dimension());

		for (int i=0 ; i<this.dimension() ; i++)
		{
			result.set(i, this.get(i) - v.get(i));
		}
		return result;
	}

	public NVector add(NVector v)
	{
		checkDimension(v);

		NVector result = new NVector(this.dimension());

		for (int i=0 ; i<this.dimension() ; i++)
		{
			result.set(i, this.get(i) + v.get(i));
		}
		return result;
	}

	public NVector scalarMult(double a)
	{
		NVector result = new NVector(this.dimension());

		for (int i=0 ; i<this.dimension() ; i++)
		{
			result.set(i, this.get(i) * a);
		}
		return result;
	}

// ---------------------------------------------------------------------
// methods that perform an operation on the this NVector
	public void substractFrom(NVector v)
	{
		checkDimension(v);

		for (int i=0 ; i<this.dimension() ; i++)
		{
			this.set(i, this.get(i) - v.get(i));
		}
	}

	public void addTo(NVector v)
	{
		checkDimension(v);

		for (int i=0 ; i<this.dimension() ; i++)
		{
			this.set(i, this.get(i) + v.get(i));
		}
	}
	public void scalarMultBy(double a)
	{
		for (int i=0 ; i<this.dimension() ; i++)
		{
			this.set(i, this.get(i) * a);
		}
	}
	public double innerproduct(NVector v)
	{
		double result=0;
		checkDimension(v);

		for (int i=0 ; i<this.dimension() ; i++)
			result += this.get(i) * v.get(i);

		return result;
	}

	public double length()
	{
		return Math.sqrt(this.innerproduct(this));
	}

// cos(phi) = innerprod(normalize(a), normalize(b))
// sin(phi) = length(crossproduct(a,b))
	public double angle(NVector v)
	{
		checkDimension(v);
		if (length()==0 && v.length()==0)
			throw new VectorException("angle with nul-vector");
		return Math.acos(this.innerproduct(v)/this.length()/v.length());
	}

	public double distance(NVector v)
	{
		checkDimension(v);
		return this.substract(v).length();
	}

	public double polar(int i)
	{
		if (Math.abs(this.length())<0.000001)
			return 0;
		else if (i==0)
		{
			if (Math.abs(this.get(0))<0.000001)
				return Math.PI/2;
			else
				return Math.atan(this.get(1)/this.get(0));
		}
		else if (i==1)
		{
			return Math.acos(this.get(2)/this.length());
		}
		else
			return 0;
	}
	public String toString()
	{
		String res = "(";

		if (_x==null)
			return "(null)";

		if (this.dimension()>0)
			res += this.get(0);

		for (int i=1 ; i<this.dimension() ; i++)
			res += "," + this.get(i);
		res += ")";

		return res;
	}
	public NVector transform(NVector translate, double scale)
	{
		return this.scalarMult(scale).add(translate);
	}

	private double[] _x;
}



/*

jordan form of matrices:

j11 j12 j13 ...
J21 j22 j23
j31 ...
...

exp(A) = S*exp(J)*S^-1

J[i,j] = 0,  1<=i<=j-2, j+1<=i<=n

J[i,i] = eigenvalue[i]
J[i,i+1] = 1, iff J[i,i]==J[i+1,i+1]

*/
