package wj.math;

import java.util.List;
import java.util.Collection;
import java.util.ArrayList;

public class Permutation {
	List _elements;
	public Permutation(int n)
	{
		_elements = new ArrayList(n);
		for (int i=0 ; i<n ; i++)
			_elements.add(new Integer(i));
	}
	public Permutation(Object[] objects)
	{
		_elements = new ArrayList(objects.length);
		for (int i=0 ; i<objects.length ; i++)
			_elements.add(objects[i]);
	}
	public Permutation(Collection c)
	{
		_elements = new ArrayList(c);
	}
	public Permutation(Permutation p)
	{
		_elements = new ArrayList(p.elements());
	}

/**

lexicografische index van een permutatie p = {p_i, i=1..n}

index(n, p) = (p_1-1)*(n-1)! + index(n-1, p\{p_1})

*/
	public FactorialNumber factorialLexicographicRank(Permutation p)
	{
		if (size()==1)
			return new FactorialNumber();

		FactorialNumber result = this.permutationWithout(p.firstElement()).factorialLexicographicRank(p.permutationWithout(p.firstElement()));

		result.setDigit(size()-1, this.elementIndex(p.firstElement()));

		return result;
	}

	public Permutation nextLexicographical(Permutation p)
	{
		int i,j;
// find  max { i |  0<=i<n && p(i) < p(i+1) }

		for (i = size()-2 ; i>=0 ; i--)
		{
			if (this.elementIndex(p.getElement(i)) <= this.elementIndex(p.getElement(i+1)))
				break;
		}
// if p is the last, the next is the first.
		if (i<0)
			return new Permutation(this);

// find min { p(j) | p(i)<p(j) & j>i }
		for (j =size()-1 ; j>i ; j--)
		{
			if (this.elementIndex(p.getElement(i)) <= this.elementIndex(p.getElement(j)))
				break;
		}
// create new permutation with i and j swapped

		Permutation newp = new Permutation(p);
		newp.exchange(i,j);

// reverse  p(i+1) .. p(n)
		for (int r=size()-1, s=i+1 ; r>s ; r--, s++)
		{
			newp.exchange(r,s);
		}

		return newp;
	}

	public Permutation nextMinimal(Permutation p)
	{
		return null;
	}

	public void exchange(int i, int j)
	{
		Object tmp = _elements.get(i);
		_elements.set(i, _elements.get(j));
		_elements.set(j, tmp);
	}
	public Object firstElement()
	{
		return _elements.get(0);
	}
	public Permutation permutationWithout(Object o)
	{
		List newelements = new ArrayList(_elements);
		newelements.remove(o);
		return new Permutation(newelements);
	}

	public int elementIndex(Object o)
	{
		return _elements.indexOf(o);
	}

	public int size()
	{
		return _elements.size();
	}
	public Object getElement(int i)
	{
		return _elements.get(i);
	}

	public String toString()
	{
		String result = "(" + firstElement();
		for (int i=1 ; i<size() ; i++)
		{
			result += ", " + getElement(i);
		}
		result += ")";

		return result;
	}
	public List elements()
	{
		return _elements;
	}
}
