import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;


public class polytope implements ActionListener {
	JTextArea _textarea;
	JComboBox _dimensionCombo;
	JComboBox _typeCombo;
/*
	JSlider _xslide;
	JSlider _yslide;
*/
	public static void main(String[] args)
	{
		new polytope();
	}
	polytope() {
		buildFrame();
	}

	JFrame buildFrame()
	{
		JFrame frame = new JFrame("polytope");

		frame.getContentPane().add(buildForm(), BorderLayout.CENTER);
		frame.addWindowListener(
				new WindowAdapter() {
					public void windowClosing(WindowEvent e) {
						System.exit(0);
					}
				}
			);
		frame.pack();
		frame.setVisible(true);

		return frame;
	}

	Component buildForm()
	{
		JPanel pane = new JPanel();
		GridBagConstraints c = new GridBagConstraints();

		pane.setBorder(BorderFactory.createEmptyBorder(30, 30, 10, 30));
		pane.setLayout(new GridBagLayout());

		c.anchor = GridBagConstraints.NORTHWEST;
		c.gridwidth = 1;
		pane.add(new JLabel("dimension"), c);

		c.anchor = GridBagConstraints.NORTHWEST;
		c.gridwidth = 1;
		pane.add(_dimensionCombo = new JComboBox(), c);
		for (int i=0 ; i<8 ; i++)
			_dimensionCombo.addItem(Integer.toString(i));
		_dimensionCombo.setEditable(true);

		c.anchor = GridBagConstraints.NORTHWEST;
		c.gridwidth = 1;
		pane.add(new JLabel("shape"), c);

		c.anchor = GridBagConstraints.NORTHWEST;
		c.gridwidth = 1;
		pane.add(_typeCombo = new JComboBox(), c);
		_typeCombo.addItem("tetra");
		_typeCombo.addItem("hexa");
		_typeCombo.addItem("octa");
/*
		c.anchor = GridBagConstraints.SOUTHWEST;
		c.gridwidth = 1;
		pane.add(_xslide = new JSlider(JSlider.HORIZONTAL), c);

		c.anchor = GridBagConstraints.SOUTHWEST;
		c.gridwidth = 1;
		pane.add(_yslide = new JSlider(JSlider.HORIZONTAL), c);
*/
		c.anchor = GridBagConstraints.NORTHWEST;
		c.gridwidth = GridBagConstraints.REMAINDER;
		JButton b = new JButton("generate");
		b.addActionListener(this);
		pane.add(b, c);

		c.anchor = GridBagConstraints.SOUTH;
		c.gridwidth = GridBagConstraints.REMAINDER;
		pane.add(new JScrollPane(_textarea = new JTextArea(25, 80)), c);
		_textarea.setLineWrap(true);


		return pane;
	}
	void print(String s)
	{
		_textarea.append(s);
	}

	public void actionPerformed(ActionEvent e)
	{
		int dimension = _dimensionCombo.getSelectedIndex();
		int type = _typeCombo.getSelectedIndex();
		PolyObject object;

		switch(type)
		{
			case 0: object = Tetraeder.make(dimension); break;
			case 1: object = Hexaeder.make(dimension); break;
			case 2: object = Octaeder.make(dimension); break;
			default: return;
		}

		print(object.toString() + "\n");
		System.out.println(object.toString());
		for (int i=0 ; i<=dimension ; i++)
		{
			print(i + " : " + object.components(i).size() + "\n");
		}
	}
}

class PolyObject implements Comparable {
	int _dimension;
	String _name;
	Vector _components;
	PolyObject _parent;

	public PolyObject(PolyObject parent, int dimension, String name)
	{
		_dimension = dimension;
		_name = name;
		_parent = parent;
		_components = new Vector();
	}

	public void add(PolyObject o)
	{
		_components.add(o);
	}

	public PolyObject find(int dimension, String name)
	{
		if (_name.equals(name) && _dimension==dimension)
			return this;

		for(Iterator i = _components.iterator() ; i.hasNext() ; )
		{
			PolyObject p = (PolyObject)i.next();
			PolyObject result = p.find(dimension, name);
			if (result!=null)
				return result;
		}
		return null;
	}

	public Vector components(int dimension)
	{
		SortedSet allcomponents = new TreeSet();
		if (dimension != _dimension)
		{
			for(Iterator i = _components.iterator() ; i.hasNext() ; )
			{
				PolyObject p = (PolyObject)i.next();
				allcomponents.addAll(p.components(dimension));
			}
		}
		else
			allcomponents.add(this);

		return new Vector(allcomponents);
	}

	public String name()
	{
		return _name;
	}
	
	public String toString()
	{
		String result = _name;
		if (_dimension==0)
			return result;

		result += "{";
		for(Iterator i = _components.iterator() ; i.hasNext() ; )
		{
			PolyObject p = (PolyObject)i.next();
			result += p.toString();
			if (i.hasNext())
				result += ", ";
		}
		result += "}";
		return result;
	}
	public int compareTo(Object o)
	{
		return name().compareTo(((PolyObject)o).name());
	}
/*
	public PolyObject findPoly(int dimension, String name)
	{
		PolyObject component = null;
		if (_parent!=null)
			component = _parent.find(dimension, name);
		if (component==null)
		{
// dit werkt niet.
			component = new getClass().newInstance(this, dimension, name);
		}
		return (Tetraeder)component;
	}
*/
}

class Tetraeder extends PolyObject {
	public Tetraeder(PolyObject parent, int dimension, String name)
	{
		super(parent, dimension, name);
// assume name.length()==dimension+1

		if (dimension==0)
			return;

		for(int i=0 ; i<=dimension ; i++)
		{
			String componentName = "";
			componentName += name.substring(0, i);
			componentName += name.substring(i+1);

			add(findPoly(dimension-1, componentName));
		}
//		System.out.println(" Tetra("+dimension+")="+toString());
	}
	public static Tetraeder make(int dimension)
	{
		return new Tetraeder(null, dimension, "ABCDEFGHIJKLMNOPQRSTUVWXYZ".substring(0,dimension+1));
	}

	public Tetraeder findPoly(int dimension, String name)
	{
		PolyObject component = null;
		if (_parent!=null)
			component = _parent.find(dimension, name);
		if (component==null)
		{
			component = new Tetraeder(this, dimension, name);
		}
		return (Tetraeder)component;
	}

}

class Hexaeder extends PolyObject {
	public Hexaeder(PolyObject parent, int dimension, String name)
	{
		super(parent, dimension, name);
// assume name.length()==2^dimension

		if (dimension==0)
			return;

		Hexaeder part1 = findPoly(dimension-1, name.substring(0,name.length()/2));
		Hexaeder part2 = findPoly(dimension-1, name.substring(name.length()/2));
		add(part1);
		add(part2);

		Vector old1 = part1.components(dimension-2);
		Vector old2 = part2.components(dimension-2);

//			ASSERT(old1.size() == old2.size());

		for (int j=0 ; j<old1.size() ; j++)
		{
			PolyObject comp1 = (PolyObject)old1.get(j);
			PolyObject comp2 = (PolyObject)old2.get(j);
			add(findPoly(dimension-1, comp1.name() + comp2.name()));
		}
//		System.out.println(" Hexa("+dimension+")="+toString());
	}
	public static Hexaeder make(int dimension)
	{
		return new Hexaeder(null, dimension, "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_$".substring(0, 1<<dimension ));
	}

	public Hexaeder findPoly(int dimension, String name)
	{
		PolyObject component = null;
		if (_parent!=null)
			component = _parent.find(dimension, name);
		if (component==null)
		{
			component = new Hexaeder(this, dimension, name);
		}		
		return (Hexaeder)component;
	}

}

class Octaeder extends PolyObject {
	public Octaeder(PolyObject parent, int dimension, String name)
	{
		super(parent, dimension, name);
// assume name.length()==2*dimension
// assume dimension>=0
//		System.out.println("Octaeder d="+dimension+" n='"+name+"'");
		if (dimension==0)
			return;

		PolyObject pointz = findPoly(0, name.substring(0,1));
		PolyObject pointZ = findPoly(0, name.substring(name.length()-1, name.length()));
		if (dimension==1)
		{
			add(pointz);
			add(pointZ);
			return;
		}
		
		PolyObject baseOcta = findPoly(dimension-1, name.substring(1, name.length()-1));
//		System.out.println("   base( d="+dimension+" n='"+name+"' ) = " + baseOcta.toString() + "   z = " + pointz.toString() + "   Z = " + pointZ.toString());

		Vector planelist = baseOcta.components(dimension-2);

		for (int j=0 ; j<planelist.size() ; j++)
		{
			PolyObject plane = (PolyObject)planelist.get(j);
			add(new Tetraeder(null, dimension-1, plane.name()+pointz.name()));
			add(new Tetraeder(null, dimension-1, plane.name()+pointZ.name()));
		}

//		System.out.println(" Octa("+dimension+")="+toString());
	}
	public static Octaeder make(int dimension)
	{
		return new Octaeder(null, dimension, "zyxwvutsrqponmlkjihgfedcbaABCDEFGHIJKLMNOPQRSTUVWXYZ".substring(26-dimension, 26+dimension));
	}

	public Octaeder findPoly(int dimension, String name)
	{
		PolyObject component = null;
		if (_parent!=null)
			component = _parent.find(dimension, name);
		if (component==null)
		{
			component = new Octaeder(this, dimension, name);
		}		
		return (Octaeder)component;
	}

}


