package wj.math.shapes;

import wj.math.NVector;
import wj.math.Matrix;

public class Plane extends Shape {
	public Plane(NVector p0, NVector p1, NVector p2)
	{
		super(null, 2, "");
		_position = p0;
		add(new Line(p0,p1));
		add(new Line(p0,p2));
	}
	public NVector direction0()
	{
		return ((Line)_components.get(0)).direction();
	}

	public NVector direction1()
	{
		return ((Line)_components.get(1)).direction();
	}
	public NVector position()
	{
		return _position;
	}


/*
 * for now only do this for this.dimension=2 and line.dimension=1
 * result will be a point
 *
 * returns null when there is no intersection
 *
 *when the intersection is outside of the area of the bounded plane,
 *there will still be an intersection.
 *
 *the plane = { line(pp0, pp1), line(pp0, pp2), ... }
 *the line = { pl0, pl1 }
 *
 * the plane can be described as x = pp0 + a*(pp1-pp0)+b*(pp2-pp0)
 * the line can be described as x = pl0 + c*(pl1-pl0)
 *
 * -> the intersection is:
 *  pp0 + a*(pp1-pp0)+b*(pp2-pp0) = pl0 + c*(pl1-pl0)
 *
 * from this a, b and c can be solved
 *
 *                                       { a }
 * { (pp1-pp0), (pp2-pp0), (pl0-pl1) } * { b } = { (pl0-pp0) }
 *                                       { c }
 *
 *------------------------------------
 * when the plane is represented by a position vector + an orthogonal base
 *    plane = {pp, p0, p1}
 * and the line is represented by  line = {lp, l0}
 *
 *  pp + a*p0 + b*p1 = lp + c*l0
 * ->
 *  {p0, p1, l0} * {a,b,c}T = { lp-pp }
 *
 *  -> a  and  b are the values wanted for the screen projection,
 *  no intermediate point is needed.
 *
 */

	public NVector intersection(Line line)
	{
		Matrix t = new Matrix(Matrix.COLUMNS,
			this.direction0(),
			this.direction1(),
			line.direction());

		NVector abc = t.solveAx_b(line.startVector().substract(this.position()));

		return line.startVector().add( line.direction().scalarMult( abc.get(2)));
	}

	NVector _position;
}
