package wj.math.shapes;

import wj.math.NVector;
import wj.math.Matrix;
import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.SortedSet;
import java.util.TreeSet;

public abstract class Shape implements Comparable {
	protected int _dimension;
	protected String _name;
	protected List _components;
	protected PolyObject _parent;

	public Shape(PolyObject parent, int dimension, String name)
	{
		_dimension = dimension;
		_name = name;
		_parent = parent;
		_components = new ArrayList();
		System.out.println(" "+toShortString()+" new("+rootPoly().toShortString()+")");
	}

	public void add(Shape o)
	{
		System.out.println(" "+toShortString()+".add("+o.toShortString()+")");
		_components.add(o);
	}

/*
 *	searches for a named Shape, in this object, and
 *	in all its children.
 */
	public Shape find(int dimension, String name)
	{
		if (_name.equals(name) && _dimension==dimension)
		{
			System.out.println(" FOUND "+toShortString()+".find("+dimension+"/"+name+")");
			return this;
		}

		if (dimension>=_dimension)
		{
			System.out.println(" NOTFOUND(dim) "+toShortString()+".find("+dimension+"/"+name+")" + toString());
			return null;
		}

		for(Iterator i = _components.iterator() ; i.hasNext() ; )
		{
			Shape p = (Shape)i.next();
			Shape result = p.find(dimension, name);
			if (result!=null)
			{
				System.out.println(" FOUND "+toShortString()+".find("+dimension+"/"+name+")");
				return result;
			}
		}
		System.out.println(" NOTFOUND "+toShortString()+".find("+dimension+"/"+name+")" + toString());
		return null;
	}

/**
 *  the polytope construction algorithms require
 *  components to be sorted.
 */
	public int compareTo(Object o)
	{
		return name().compareTo(((Shape)o).name());
	}
	public List components(int dimension)
	{
		SortedSet allcomponents = new TreeSet();
		if (dimension != _dimension)
		{
			for(Iterator i = _components.iterator() ; i.hasNext() ; )
			{
				Shape p = (Shape)i.next();
				allcomponents.addAll(p.components(dimension));
			}
		}
		else
			allcomponents.add(this);

		return new ArrayList(allcomponents);
	}

	public String name()
	{
		return _name;
	}
	
	public String toShortString()
	{
		return getClass().getName()+"::"+name();
	}
	public String toString()
	{
		String result = name();
		if (_dimension==0)
			return result;

		result += "{";
		for(Iterator i = _components.iterator() ; i.hasNext() ; )
		{
			Shape p = (Shape)i.next();
			result += p.toString();
			if (i.hasNext())
				result += ", ";
		}
		result += "}";
		return result;
	}

	public boolean contains(Shape o)
	{
		if (this.equals(o))
			return true;

		for(Iterator i = _components.iterator() ; i.hasNext() ; )
		{
			Shape p = (Shape)i.next();
			if (p.contains(o))
				return true;
		}
		return false;
	}
	public int dimension()
	{
		return _dimension;
	}

	public int spaceDimension()
	{
		if (_parent==null)
			return dimension();
		else
			return _parent.dimension();
	}
	public boolean isRoot()
	{
		return _parent==null;
	}
	public PolyObject rootPoly()
	{
		if (isRoot())
			return (PolyObject)this;
		else
			return _parent.rootPoly();
	}
}

