package wj.math.shapes;

import wj.math.NVector;

public class Tetraeder extends PolyObject {
	public Tetraeder(PolyObject parent, int dimension, String name)
	{
		super(parent, dimension, name);
// assume name.length()==dimension+1

		if (dimension==0)
			return;

		for(int i=0 ; i<=dimension ; i++)
		{
			String componentName = "";
			componentName += _name.substring(0, i);
			componentName += _name.substring(i+1);

			add(findPoly(dimension-1, componentName));
		}
	}
	public static Tetraeder make(int dimension)
	{
		return new Tetraeder(null, dimension, "ABCDEFGHIJKLMNOPQRSTUVWXYZ".substring(0,dimension+1));
	}

	protected PolyObject newInstance(PolyObject parent, int dimension, String name)
	{
		return new Tetraeder(parent, dimension, name);
	}

/*
 *	calculating coordinates for a tetraeder:
 *
 *	for a (n-1)-dimensional Tetraeder: take unit vectors of a n dimensional space.
 *
 *	translate it to the origin ( substract 1/n from all coordinates )
 *
 *	decompose in Q*R  ( Q = rotation matrix )
 *
 *	-> coordinates are in the R matrix.
 *
 *{lower-left triangle}
 *Rn[i,j] = 0   (1<=j<i<=n)
 *
 *{diagonal}
 *Rn[i,j] = sqrt(n-i)/sqrt(n-i+1)   (1<=i<=n-1, i=j)
 *Rn[i,j] = 0    (i=j=n)
 *
 *{upper-right triangle}
 *Rn[i,j] = - 1/sqrt((n-i)*(n+1-i))   (1<=i<j<=n)
 *
 *----------------------------
 *voor een m-dimensionale tetraeder wordt dat dan dus:
 *
 *Tm_j[i] = 0                       (1<=j<i<=m)
 *Tm_j[i] = sqrt(m+1-i)/sqrt(m+2-i)   (1<=i<=m, i=j)
 *Tm_j[i] = - 1/sqrt((m+1-i)*(m+2-i))   (1<=i<j<=m+1)
 *
 */
	protected NVector getCoordinates(String name) throws RuntimeException
	{
		NVector vector = new NVector(spaceDimension());
		char c = name.charAt(0);
		int j = 1+c-'A';
		int m = spaceDimension();

		if (j<0 || j>m+1)
			throw new RuntimeException("invalid tetraeder point name \'"+name+"\'");

		if (j<=m)
			vector.set(j-1, Tetraeder.SCALE * Math.sqrt(m+1-j)/Math.sqrt(m+2-j));

		for (int i=1 ; i<j ; i++)
			vector.set(i-1, -Tetraeder.SCALE/Math.sqrt((m+1-i)*(m+2-i)));

		return vector;
	}
	private final static double SCALE=Math.sqrt(0.5);
}
