package wj.space;

import wj.math.Matrix;
import wj.math.NVector;
import wj.math.shapes.Point;
import wj.math.shapes.Line;
import wj.math.shapes.Plane;
import java.util.Iterator;
import java.awt.Graphics;

// I should be drawing onto a graphics device which knows
// how large it is.
public class NSpace {
	public NSpace(int _dimension)
	{
		_dimension = _dimension;
		_viewPoint = null;
		_projectionPlane = null;
	}

	public void drawPoint(Graphics g, NVector point)
	{
		int[] p = calculateProjection(point);

//		System.out.println("   Drawing point "
//				+ point+"@["+p[0]+","+p[1]+"]");

		g.drawOval(_width+p[0], _height-p[1], _dotSize, _dotSize);
	}

	public void drawLine(Graphics g, NVector point1, NVector point2)
	{
		int[] p1 = calculateProjection(point1);
		int[] p2 = calculateProjection(point2);

//		System.out.println("   Drawing line "
//				+ point1+"@["+p1[0]+","+p1[1]+"] - "
//				+ point2+"@["+p2[0]+","+p2[1]+"]");

		g.drawLine(_width+p1[0], _height-p1[1], _width+p2[0], _height-p2[1]);
	}


/***********************************************************************
 *	normalize dir0 and dir1 as follows:
 *
 *  n0 = dir0/norm(dir0)
 *  n1 = dir1/norm(dir1)
 *  n1 = n1 - n1.n0 * n0
 *
 * calculate x and y such that
 *
 * point = x*dir0+y*dir1
 *
 *
 * where plane = { dir0, dir1 }
 *
 * { dir0, dir1 } * { x } = { point }
 *                  { y }
 *

 */

	public void setViewPoint(NVector point, double _scale)
	{
		_viewPoint = point;
	}

	public void setProjectionPlane(Plane plane)
	{
		_projectionPlane = plane;

		NVector n0 = plane.direction0().scalarMult(1/plane.direction0().length());
		NVector n1 = plane.direction1().scalarMult(1/plane.direction1().length());

		n1 = n1.substract(n0.scalarMult(n1.innerproduct(n0)));

		_projectionMatrix = new Matrix(Matrix.COLUMNS,
			n0, n1);
	}

	private int[] calculateProjection(NVector point)
	{
		NVector proj = _projectionPlane.intersection(new Line(null, new Point(point), new Point(_viewPoint), "viewline"));
		NVector result = _projectionMatrix.solveAx_b(proj);

//	the choice of 'x' and 'y' is arbitrary here, since it is not known
//	in what orientation the observer will be looking at the projection plane
//	from the projection point

		return new int[]{(int)(result.get(1)*_scale), (int)(result.get(0)*_scale)};

	}

	public void setSize(int w, int h)
	{
		_width = w;
		_height = h;
	}

	int _dotSize = 10;

	NVector _viewPoint;
	double _scale;
	Plane _projectionPlane;

	Matrix _projectionMatrix;

	int _dimension;

// of actual screen
	int _width;
	int _height;
}

/*

ik wil bewijzen dat er vanuit een punt p maar 1 loodlijn
op een vlak V kan vallen.

vlak V = v0 + d0*x + d1 * y

punt p

loodlijn: (p',p)

p' = V(xp, yp)

loodlijn: (p-p') . d0 = (p-p') . d1 = 0

(p -  v0 + d0*xp + d1 * yp ) . d0 = 0


-p.d0 +  v0.d0 + d0.d0*xp + d1.d0 * yp = 0

-p.d1 +  v0.d1 + d0.d1*xp + d1.d1 * yp = 0

(d0.d0   d1.d0)   (xp)   (p-v0).d0
(d0.d1   d1.d1) * (yp) = (p-v0).d1

Mt[d0,d1]*M[d0,d1] * (x,y) = Mt[d0,d1]*(v0-p)


bij orthonormale basis:
  xp = (p-v0).d0
  yp = (p-v0).d1

*/
